@@ -173,6 +173,7 @@ class AmiPush(AmiTask, RHSMClientService, AWSPublishService, CollectorService):
 
         return "-".join(parts)
 
+    # pylint:disable=too-many-locals
     def upload(self, push_item):
         """
         Uploads and imports a disk image to AWS. If ship is not True, the image
@@ -230,6 +231,7 @@ class AmiPush(AmiTask, RHSMClientService, AWSPublishService, CollectorService):
             "volume_type": push_item.volume,
             "billing_products": push_item.billing_codes.codes,
             "accounts": accounts,
+            "snapshot_account_ids": self.args.snapshot_account_ids,
             "sriov_net_support": push_item.sriov_net_support,
             "ena_support": push_item.ena_support or False,
         }
@@ -359,7 +361,7 @@ class AmiPush(AmiTask, RHSMClientService, AWSPublishService, CollectorService):
                 # break statement is not covered in py38
                 # https://github.com/nedbat/coveragepy/issues/772
                 break  # pragma: no cover
-            attr.evolve(push_item, state=state)
+            dest_data["push_item"] = attr.evolve(push_item, state=state)
             dest_data["state"] = state
             dest_data["image_id"] = image_id
             dest_data["image_name"] = image_name
@@ -372,10 +374,19 @@ class AmiPush(AmiTask, RHSMClientService, AWSPublishService, CollectorService):
             if isinstance(obj, (datetime.datetime, datetime.date)):
                 return obj.strftime("%Y%m%d")
 
+        mod_result = []
         for result in results:
-            result["push_item"] = attr.asdict(result["push_item"])
+            res_dict = attr.asdict(result["push_item"])
+            # dict can't be modified during iteration.
+            # so iterate over list of keys.
+            for key in list(res_dict):
+                if res_dict[key] is None:
+                    del res_dict[key]
+            res_dict["ami"] = result["image_id"]
+            res_dict["name"] = result["image_name"]
+            mod_result.append(res_dict)
 
-        metadata = json.dumps(results, default=convert, indent=2, sort_keys=True)
+        metadata = json.dumps(mod_result, default=convert, indent=2, sort_keys=True)
         return self.collector.attach_file("images.json", metadata).result()
 
     def add_args(self):
@@ -439,6 +450,16 @@ class AmiPush(AmiTask, RHSMClientService, AWSPublishService, CollectorService):
             default=4,
         )
 
+        group.add_argument(
+            "--snapshot-account-ids",
+            help="Comma separated list of account ids to give snapshot "
+            "creation permissions to if a new snapshot is created as "
+            "part of the image push.",
+            action=SplitAndExtend,
+            split_on=",",
+            default=["679593333241", "684062674729", "425685993791", "582767206473"],
+        )
+
     def run(self):
         # verify push_items
         if not self.items_in_metadata_service():
@@ -461,9 +482,19 @@ class AmiPush(AmiTask, RHSMClientService, AWSPublishService, CollectorService):
         for f_out in to_await:
             result.extend(f_out.result())
 
+        # process result for failures
+        failed = False
+        for r in result:
+            if not r.get("image_id"):
+                failed = True
+
         # send to collector
         LOG.info("Collecting results")
         self.collect_push_result(result)
+
+        if failed:
+            self.fail("AMI upload failed")
+
         LOG.info("AMI upload completed")
 
 
